#ifndef __SERVER_INTF_H_
#define __SERVER_INTF_H_

#include <stdbool.h>
#include <stddef.h>

#include "ds485n-packet.h"

struct dS485_server;
struct _dS485dClient;

/**
 * Packet filter bits
 */
#define DS485_FILTER_ALL_RESPONSES   0x01

/**
 * set loglevel via environment
 * see man syslog.h, valid args are 0(emerg)-7(debug)
 * default is 5(notice)
 */
#define DS485_NETLIB_DEBUG "DS485_NETLIB_DEBUG"

/**
 * if environment variable is set, log output is redirected to syslog
 */
#define DS485_SYSLOG "DS485_SYSLOG"

struct dS485_server_callbacks
{
    /**
     * client sent a packet for forwarding
     */
    int (*receive_data)(struct dS485_server *server, const ds485_container_t* packet);

    /**
     * client sent a long packet for forwarding
     */
    int (*receive_long_data)(struct dS485_server *server, const ds485n_packet_t* packet);

    /**
     * Fill devices array with dsuid of all devices
     */
    int (*get_bus_members)(struct dS485_server *server, dsuid_t *devices, int length);

    /**
     * get server address
     */
    dsuid_t (*get_dsuid)(struct dS485_server *server);

    /**
     * current state of the server
     */
    bus_state_t (*get_bus_state)(struct dS485_server *server);

    int (*restart_bus)(struct dS485_server *server);

    /**
     * promiscious mode, forwared all packets
     */
    int (*set_packetfilter)(struct dS485_server *server, uint8_t rxfilter);

    /**
     * internal/testing: trigger when server is ready to
     * accept clients, don't use
     */
    int (*status_change)(struct dS485_server *server, int status);
};

/*
 * Generic parts
 */
void ds485_server_emit_bus_change(struct dS485_server *server, dsuid_t dsuid, int
                                  flags);
void ds485_server_emit_bus_state(struct dS485_server *server, bus_state_t state);

void ds485_server_reply(struct dS485_server *server,
                        const ds485_container_t *container);
void ds485_server_reply_long(struct dS485_server *server,
                             const ds485n_packet_t *packet);

struct dS485_server *ds485_server_init(struct dS485_server_callbacks *cb, bool upperPath);
void ds485_server_cleanup(struct dS485_server *server);

int ds485_server_listen(struct dS485_server* server, int port, bool listenGlobal);
int ds485_server_listen2(struct dS485_server* server, int port, bool listenGlobal, int maxClients);
void ds485_server_abort(struct dS485_server *server);


#endif
