#ifndef __DS485N_PACKET_H__
#define __DS485N_PACKET_H__

/*
 * WARNING
 * This file is processed by SWIG in dsm-api.
 * Always rebuild dsm-api after changing this file
 * Some problems occur only during runtime, verify using
 * https://git.digitalstrom.org/dsmduo/dsmduo-tools/blob/master/ds.lua
 */

#ifndef SWIG
/*  SWIG bombs on __attribute__((packed)) */
#define ATTRIBUTE_PACKED __attribute__((packed))
#else
#define ATTRIBUTE_PACKED
#endif

#include <digitalSTROM/dsuid.h>

#ifndef DS485_CONST_H_MOVE_SYMBOLS
#define DS485_CONST_H_MOVE_SYMBOLS
/*
 * Message Declarations
 */
enum DS485MessageSizes {
    DS485_MIN_DATA_PAYLOAD_SIZE = 3,        /* transaction ID + message ID + modifier/error code */
    DS485_HEADER_CRC_SIZE       = 6,        /* start + destination + source + command/length + crc16 */
    DS485_SHORT_PAYLOAD_SIZE    = 15,
    DS485_LONG_PAYLOAD_SIZE     = 128,
    DS485_SHORT_MESSAGE_SIZE    = (DS485_SHORT_PAYLOAD_SIZE+DS485_HEADER_CRC_SIZE),
    DS485_LONG_MESSAGE_SIZE     = (DS485_LONG_PAYLOAD_SIZE+DS485_HEADER_CRC_SIZE)
};
#endif /* DS485_CONST_H_MOVE_SYMBOLS */


#ifndef DS485_H_MOVE_SYMBOLS
#define DS485_H_MOVE_SYMBOLS

#define DS485_MAX_PAYLOAD_SIZE 127
#define DS485_MAX_PAYLOAD_SIZE_RAW DS485_MAX_PAYLOAD_SIZE + 1
#define DS485_CONTAINER_DATA_LENGTH DS485_MAX_PAYLOAD_SIZE
#define DS485_CONTAINER_SIZE (sizeof(ds485_container_t) - (DS485_CONTAINER_DATA_LENGTH))

/**
 * Status des dS485 Bus
 */
typedef enum {
    DS485_STATE_ISOLATED = 1,
    DS485_STATE_JOIN,
    DS485_STATE_CONNECTED,
    DS485_STATE_ACTIVE,
    DS485_STATE_DISCONNECTED,
} bus_state_t;

/**
 * Typ des dS485Packets
 */
typedef enum {
    DS485_CONTAINER_REQUEST = 1,
    DS485_CONTAINER_RESPONSE,
    DS485_CONTAINER_EVENT
} container_type_t;

typedef enum {
    DS485_FLAG_NONE = 1,
    DS485_FLAG_BROADCAST = 2,
} container_flags_t;

/**
 * dS485 API Daten Container
 */
typedef struct ds485_container {
    dsuid_t destinationId;
    dsuid_t sourceId;
    uint32_t containerType;
    uint32_t containerFlags;
    uint8_t length;
    union {
        struct {
            unsigned char transactionId;
            unsigned char data[DS485_MAX_PAYLOAD_SIZE];
        };
        unsigned char raw_data[DS485_MAX_PAYLOAD_SIZE_RAW];
    };
} ATTRIBUTE_PACKED ds485_container_t;

#endif /* DS485_H_MOVE_SYMBOLS */

enum DS485NetlibPacketSizes {
    DS485N_PACKET_PAYLOAD_SIZE     = 8600,
    DS485N_PACKET_MESSAGE_SIZE     = (DS485N_PACKET_PAYLOAD_SIZE+DS485_HEADER_CRC_SIZE)
};

enum DS485NetlibPacketOffset {
    DS485N_PACKET_START_OFFSET     = 0,
    DS485N_PACKET_DEST_OFFSET      = 1,
    DS485N_PACKET_SRC_OFFSET       = 2,
    DS485N_PACKET_CL_OFFSET_LOW    = 3,
    DS485N_PACKET_CL_OFFSET_HIGH   = 4,
    DS485N_PACKET_DATA_OFFSET      = 5,
};

#define DS485N_PACKET_MAX_PAYLOAD_SIZE      DS485N_PACKET_PAYLOAD_SIZE - 1
#define DS485N_PACKET_MAX_PAYLOAD_SIZE_RAW  DS485N_PACKET_MAX_PAYLOAD_SIZE + 1
#define DS485N_HDR_LEN                      (offsetof(ds485n_packet_t, data))

typedef struct ds485n_packet {
    dsuid_t destinationId;
    dsuid_t sourceId;
    uint16_t dummy; // explictly needed now as we pack the struct; without packing compilers did 32-bit align here
    uint32_t containerType; /* legacy */ // using uint32_t, enum type is dangerous because it could be 1,2 or 4 bytes depending on compiler
    uint16_t length; // TODO: this is only works on a little-endian architecture (x86, most ARM). Should be made endian-independent
    union {
        struct {
            unsigned char transactionId;
            unsigned char data[DS485N_PACKET_MAX_PAYLOAD_SIZE];
        } ATTRIBUTE_PACKED;
        unsigned char raw_data[DS485N_PACKET_MAX_PAYLOAD_SIZE_RAW];
    } ATTRIBUTE_PACKED;
} ATTRIBUTE_PACKED ds485n_packet_t;

#endif /* __DS485N_PACKET_H__ */
